package net.sf.javaprinciples.data.transformer;

import java.util.Collection;

import net.sf.javaprinciples.core.UnexpectedException;

/**
 * Mapper to handle the processing of an existing Collection of Objects.
 * Expects an output of type Collection.
 * Adds an additional instance to the collection and passes it to the mapper associations
 * to map. If attributes are mapped then the instance is added to the collection.
 *
 * @author Kay Chevalier
 */
public class AdditionalInstanceOutputMapper<T,U> extends ModelElementMapper implements Mapper<T, U>
{

    @Override
    public void map(T input, U output)
    {
        Object outputAttribute = null;
        String attributeName =  ModelElementMapperHelper.getPropertyNameFromElement(getDestinationModelElement());

        Object outputInstance = ModelElementMapperHelper.instantiateObjectFromModel(objectTypeMapper,
                getDestinationModelElement(), store, output, attributeName);
        outputAttribute = objectTypeMapper.getAttributeFromObject(output, attributeName);

        if (!(outputAttribute instanceof Collection<?>))
        {
            throw new UnexpectedException("AdditionalInstanceOutputMapper expects an ArrayList for the output");
        }

        boolean attributesMapped = processMapperAssociations(input, outputInstance);
        if (attributesMapped)
        {
            ((Collection) outputAttribute).add(outputInstance);
        }
    }
}