package net.sf.javaprinciples.data.transformer;

import java.util.List;

import net.sf.javaprinciples.core.UnexpectedException;
import net.sf.jcc.model.parser.uml2.ElementStore;
import net.sf.jcc.model.parser.uml2.ModelElement;

/**
 * Inject the remaining attributes of a ComplexMapper.
 *
 * @author Warwick Slade
 */
public class DelegateMapperBuilder<T,U> extends ModelElementFactory implements ModelElementMapperBuilder<DelegateMapper>
{
    private MapperFactory mapperFactory;

    @Override
    public void build(DelegateMapper mapperModelElement, TransformerFactory.DIRECTION_TYPE direction)
    {
        ElementStore store = mapperModelElement.store;
        ModelElement modelElement = mapperModelElement.getModelElement();

        ModelElement parentModelElement = store.get(modelElement.getParentGuid());
        List<Mapper> mappers = retrieveMappersFromAssociations(parentModelElement, store, direction);

        if (mappers.size() != 1)
        {
            throw new UnexpectedException("");
        }

        mapperModelElement.setMapper(mappers.get(0));
        log.info("Creating Delegate Mapper: {}, {} ",
                new Object[] {
                        mapperModelElement.getClass().getName(),
                        mapperModelElement.getModelElement().getGuid()
                }
        );

    }

    protected ModelElementMapper<T,U>  createModelElementMapper(String mapperGuid, ElementStore store,
                                                                TransformerFactory.DIRECTION_TYPE direction)
    {
        return (ModelElementMapper<T,U>)mapperFactory.createMapper(mapperGuid, store, direction);
    }

    public void setMapperFactory(MapperFactory mapperFactory)
    {
        this.mapperFactory = mapperFactory;
    }
}
