package net.sf.javaprinciples.data.transformer.attributemapper;

import java.util.Collection;
import java.util.List;

import org.springframework.beans.BeanWrapperImpl;

import net.sf.javaprinciples.data.transformer.JaxbMapper;
import net.sf.javaprinciples.data.transformer.SourceDestinationAttributeMapper;
import net.sf.jcc.model.parser.uml2.ModelElement;

/**
 * Where the input value is not a list add that value to a list.
 *
 * @author Warwick Slade
 */
public class ListValueMapper extends SourceDestinationAttributeMapper
{
    private JaxbMapper jaxbMapper;

    /**
     * For each model element find the collection and add the value to it.
     *
     * @param modelElements - a list of ModelElements to retrieve the parent object attribute name from.
     * @param sourceAttributeValue - the value to be set on the parent object
     * @param parentObject - the object to set the attribute value on.
     */
    @Override
    public void setAttributeOnParentObject(List<ModelElement> modelElements, Object sourceAttributeValue, Object parentObject)
    {
        for (ModelElement modelElement : modelElements)
        {
            String destinationAttributeName = getAttributeName(modelElement);

            Class targetClass = jaxbMapper.determineModelTypeFromTarget(destinationAttributeName, parentObject);

            destinationAttributeName = jaxbMapper.nameToIdentifier(destinationAttributeName, parentObject);

            Collection collection = (Collection)new BeanWrapperImpl(parentObject).getPropertyValue(destinationAttributeName);

            if (targetClass != null && sourceAttributeValue != null)
            {
                // Cooerce the value
                sourceAttributeValue = coerce(sourceAttributeValue, targetClass);
            }

            collection.add(sourceAttributeValue);
        }
    }

    protected <T> T coerce(Object source, Class<T> targetClass)
    {
        if (targetClass == Long.class)
        {
            if (source instanceof Long)
            {
                return (T)source;
            }
            if (source instanceof String)
            {
                return (T)new Long(((String)source));
            }
            if (source instanceof Integer)
            {
                return (T)new Long((Integer)source);
            }
        }
        return (T)source;
    }

    public void setJaxbMapper(JaxbMapper jaxbMapper)
    {
        this.jaxbMapper = jaxbMapper;
    }
}
