package net.sf.javaprinciples.data.transformer.attributemapper;

import java.util.List;

import net.sf.javaprinciples.core.UnexpectedException;
import net.sf.javaprinciples.data.transformer.SourceDestinationAttributeMapper;
import net.sf.jcc.model.parser.uml2.ModelElement;

/**
 * Uses simple logic to determine a value.
 *
 * @author Warwick Slade
 */
public class LogicAttributeMapper extends SourceDestinationAttributeMapper
{
    // Logic :
    public enum Logic
    {
        And,
        Or
    }

    private Logic logic;

    @Override
    public Object getAttributeFromObject(List<ModelElement> modelElements, Object parentObject)
    {
        boolean[] values = new boolean[modelElements.size()];

        int i = 0;
        for (ModelElement modelElement : modelElements)
        {
            String attributeName = getAttributeName(modelElement);
            Object attributeValue = objectTypeMapper.getSourceAttribute(parentObject, attributeName);

            values[i++] = objectToBoolean(attributeValue);
        }

        // check logic
        for (boolean value : values)
        {
            if (logic == Logic.And)
            {
                // Once we have a false we are done
                if (!value)
                {
                    return booleanToObject(false);
                }
            }
            else
            {
                // Once we have a true we are done
                if (value)
                {
                    return booleanToObject(true);
                }

            }
        }

        boolean result = logic == Logic.And ? true : false;

        return booleanToObject(result);

    }

    public void setLogic(Logic logic)
    {
        this.logic = logic;
    }

    private Object booleanToObject(boolean result)
    {
        return result ? "Y" : "N";
    }

    private boolean objectToBoolean(Object attributeValue)
    {
        if (attributeValue == null)
        {
            return false;
        }
        if (attributeValue instanceof Boolean)
        {
            return (Boolean)attributeValue;
        }
        if ("Y".equals(attributeValue))
        {
            return true;
        }
        if ("N".equals(attributeValue))
        {
            return false;
        }
        throw new UnexpectedException("Invalid true false value:" + attributeValue);
    }
}
